@@ -6,17 +6,12 @@ Font::TTF::Cmap - Character map table
 
 =head1 DESCRIPTION
 
-Looks after the character map. The primary structure used for handling a cmap
-is the L<Font::TTF::Segarr> which handles the segmented arrays of format 4 tables,
-and in a simpler form for format 0 tables.
+Looks after the character map. For ease of use, the actual cmap is held in
+a hash against codepoint. Thus for a given table:
 
-Due to the complexity of working with segmented arrays, most of the handling of
-such arrays is via methods rather than via instance variables.
-
-One important feature of a format 4 table is that it always contains a segment
-with a final address of 0xFFFF. If you are creating a table from scratch this is
-important (although L<Font::TTF::Segarr> can work quite happily without it).
+    $gid = $font->{'cmap'}{'Tables'}[0]{'val'}{$code};
 
+Note that C<$code> should be a true value (0x1234) rather than a string representation.
 
 =head1 INSTANCE VARIABLES
 
@@ -116,8 +111,6 @@ sub read
             my ($j) = 0;
             $fh->read($dat, 256);
             $s->{'val'} = {map {$j++; ($_ ? ($j - 1, $_) : ())} unpack("C*", $dat)};
-            $s->{'Start'} = 0;
-            $s->{'Num'} = 256;
         } elsif ($form == 6)
         {
             my ($start, $ecount);
@@ -125,8 +118,6 @@ sub read
             $fh->read($dat, 4);
             ($start, $ecount) = unpack("n2", $dat);
             $fh->read($dat, $ecount << 1);
-            $s->{'Start'} = $start;
-            $s->{'Num'} = $ecount;
             $s->{'val'} = {map {$start++; ($_ ? ($start - 1, $_) : ())} unpack("n*", $dat)};
         } elsif ($form == 2)
         {
@@ -144,7 +135,6 @@ sub read
                 $delta = unpack("n", substr($dat, ($j << 1) + ($num << 2) + 2, 2));
                 $delta -= 65536 if $delta > 32767;
                 $range = unpack("n", substr($dat, ($j << 1) + $num * 6 + 2, 2));
-                $s->{'Start'} = $start unless $j;
                 for ($k = $start; $k <= $end; $k++)
                 {
                     if ($range == 0)
@@ -156,8 +146,7 @@ sub read
                     $s->{'val'}{$k} = $id if ($id);
                 }
             }
-            $s->{'Num'} = 0x10000;               # always ends here
-        } elsif ($form == 8 || $form == 10)
+        } elsif ($form == 8 || $form == 12)
         {
             if ($form == 8)
             {
@@ -175,7 +164,7 @@ sub read
                 for ($k = $start; $k <= $end; $k++)
                 { $s->{'val'}{$k} = $s++; }
             }
-        } elsif ($form == 12)
+        } elsif ($form == 10)
         {
             $fh->read($dat, 8);
             ($start, $num) = unpack("N2", $dat);
@@ -190,8 +179,8 @@ sub read
 
 =head2 $t->ms_lookup($uni)
 
-Given a Unicode value in the MS table (Platform 3, Encoding 1) locates that
-table and looks up the appropriate glyph number from it.
+Finds a Unicode table, giving preference to the MS one, and looks up the given
+Unicode codepoint in it to find the glyph id.
 
 =cut
 
@@ -206,7 +195,7 @@ sub ms_lookup
 
 =head2 $t->find_ms
 
-Finds the Microsoft Unicode table and sets the C<mstable> instance variable
+Finds the a Unicode table, giving preference to the Microsoft one, and sets the C<mstable> instance variable
 to it if found. Returns the table it finds.
 
 =cut
@@ -250,20 +239,24 @@ sub out
 
     for ($i = 0; $i < $self->{'Num'}; $i++)
     { $fh->print(pack("nnN", $self->{'Tables'}[$i]{'Platform'}, $self->{'Tables'}[$i]{'Encoding'}, 0)); }
-    
+
     for ($i = 0; $i < $self->{'Num'}; $i++)
     {
         $s = $self->{'Tables'}[$i];
         @keys = sort {$a <=> $b} keys %{$s->{'val'}};
         $s->{' outloc'} = $fh->tell();
-        $fh->print(pack("n3", $s->{'Format'}, 0, $s->{'Ver'}));       # come back for length
+        if ($s->{'Format'} < 8)
+        { $fh->print(pack("n3", $s->{'Format'}, 0, $s->{'Ver'})); }       # come back for length
+        else
+        { $fh->print(pack("n2N2", $s->{'Format'}, 0, 0, $s->{'Ver'})); }
+            
         if ($s->{'Format'} == 0)
         {
             $fh->print(pack("C256", @{$s->{'val'}}{0 .. 255}));
         } elsif ($s->{'Format'} == 6)
         {
-            $fh->print(pack("n2", $s->{'Start'}, $s->{'Num'}));
-            $fh->print(pack("n*", @{$s->{'val'}}{$s->{'Start'} .. $s->{'Start'} + $s->{'Num'} - 1}));
+            $fh->print(pack("n2", $keys[0], $keys[-1] - $keys[0] + 1));
+            $fh->print(pack("n*", @{$s->{'val'}}{$keys[0] .. $keys[-1]}));
         } elsif ($s->{'Format'} == 2)
         {
         } elsif ($s->{'Format'} == 4)
@@ -273,7 +266,7 @@ sub out
 
             push(@keys, 0xFFFF) unless ($keys[-1] == 0xFFFF);
             $newseg = 1; $num = 0;
-            for ($j = 0; $j <= $#keys; $j++)
+            for ($j = 0; $j <= $#keys && $keys[$j] <= 0xFFFF; $j++)
             {
                 $v = $s->{'val'}{$keys[$j]};
                 if ($newseg)
@@ -323,11 +316,45 @@ sub out
                 next if ($range[$j] == 0);
                 $fh->print(pack("n*", @{$s->{'val'}}{$starts[$j] .. $ends[$j]}));
             }
+        } elsif ($s->{'Format'} == 8 || $s->{'Format'} == 12)
+        {
+            my (@jobs, $start, $current, $curr_glyf, $map);
+            
+            $map = "\000" x 8192;
+            foreach $j (@keys)
+            {
+                if ($j > 0xFFFF)
+                {
+                    if (defined $s->{'val'}{$j >> 16})
+                    { $s->{'Format'} = 12; }
+                    vec($map, $j >> 16, 1) = 1;
+                }
+                if ($j != $current + 1 || $s->{'val'}{$j} != $curr_glyf + 1)
+                {
+                    push (@jobs, [$start, $current, $curr_glyf]) if (defined $start);
+                    $start = $j; $current = $j; $curr_glyf = $s->{'val'}{$j};
+                }
+            }
+            $fh->print($map) if ($s->{'Format'} == 8);
+            $fh->print(pack('N', $#jobs + 1));
+            foreach $j (@jobs)
+            { $fh->print(pack('N3', @{$j})); }
+        } elsif ($s->{'Format'} == 10)
+        {
+            $fh->print(pack('N2', $keys[0], $keys[-1] - $keys[0] + 1));
+            $fh->print(pack('n*', $s->{'val'}{$keys[0] .. $keys[-1]}));
         }
 
         $loc = $fh->tell();
-        $fh->seek($s->{' outloc'} + 2, 0);
-        $fh->print(pack("n", $loc - $s->{' outloc'}));
+        if ($s->{'Format'} < 8)
+        {
+            $fh->seek($s->{' outloc'} + 2, 0);
+            $fh->print(pack("n", $loc - $s->{' outloc'}));
+        } else
+        {
+            $fh->seek($s->{' outloc'} + 4, 0);
+            $fh->print(pack("N", $loc - $s->{' outloc'}));
+        }
         $fh->seek($base_loc + 8 + ($i << 3), 0);
         $fh->print(pack("N", $s->{' outloc'} - $base_loc));
         $fh->seek($loc, 0);
@@ -336,9 +363,31 @@ sub out
 }
 
 
+=head2 $t->XML_element($context, $depth, $name, $val)
+
+Outputs the elements of the cmap in XML. We only need to process val here
+
+=cut
+
+sub XML_element
+{
+    my ($self, $context, $depth, $k, $val) = @_;
+    my ($fh) = $context->{'fh'};
+    my ($i);
+
+    return $self if ($k eq 'LOC');
+    return $self->SUPER::XML_element($context, $depth, $k, $val) unless ($k eq 'val');
+
+    $fh->print("$depth<mappings>\n");
+    foreach $i (sort {$a <=> $b} keys %{$val})
+    { $fh->printf("%s<map code='%04X' glyph='%s'/>\n", $depth . $context->{'indent'}, $i, $val->{$i}); }
+    $fh->print("$depth</mappings>\n");
+    $self;
+}
+
 =head2 @map = $t->reverse([$num])
 
-Returns a reverse map of the table of given number or the Microsoft
+Returns a reverse map of the table of given number or the Unicode
 cmap. I.e. given a glyph gives the Unicode value for it.
 
 =cut
@@ -354,6 +403,22 @@ sub reverse
     @res;
 }
 
+
+=head2 is_unicode($index)
+
+Returns whether the table of a given index is known to be a unicode table
+(as specified in the specifications)
+
+=cut
+
+sub is_unicode
+{
+    my ($self, $index) = @_;
+    my ($pid, $eid) = ($self->{'Tables'}[$index]{'Platform'}, $self->{'Tables'}[$index]{'Encoding'});
+
+    return ($pid == 3 || $pid == 0 || ($pid == 2 && $eid == 1));
+}
+
 1;
 
 =head1 BUGS
