@@ -192,6 +192,7 @@ is required for each component.
 use strict;
 use vars qw(%fields);
 use Font::TTF::Utils;
+use Font::TTF::Table;
 
 sub init
 {
@@ -219,7 +220,7 @@ sub new
 
     bless $self, $class;
     foreach $p (keys %parms)
-    { $self->{"$p"} = $parms{$p}; }
+    { $self->{" $p"} = $parms{$p}; }
     init unless defined $fields{'xMin'};
     $self;
 }
@@ -236,14 +237,14 @@ its constituent structures. Use read_dat for this.
 sub read
 {
     my ($self) = @_;
-    my ($fh) = $self->{'INFILE'};
+    my ($fh) = $self->{' INFILE'};
     my ($dat);
 
     return $self if $self->{' read'};
     $self->{' read'} = 1;
-    $fh->seek($self->{'LOC'} + $self->{'BASE'}, 0);
-    $fh->read($self->{'DAT'}, $self->{'LEN'});
-    TTF_Read_Fields($self, $self->{'DAT'}, \%fields);
+    $fh->seek($self->{' LOC'} + $self->{' BASE'}, 0);
+    $fh->read($self->{' DAT'}, $self->{' LEN'});
+    TTF_Read_Fields($self, $self->{' DAT'}, \%fields);
     $self;
 }
 
@@ -263,7 +264,7 @@ sub read_dat
 
     return $self if $self->{' read'} > 1;
     $self->read unless $self->{' read'};
-    $dat = $self->{'DAT'};
+    $dat = $self->{' DAT'};
     $fp = 10;
     $num = $self->{'numberOfContours'};
     if ($num > 0)
@@ -391,12 +392,81 @@ sub out
 
     $self->read unless $self->{' read'};
     $self->update if $self->{' isDirty'};
-    $fh->print($self->{'DAT'});
-    $self->{'OUTLEN'} = length($self->{'DAT'});
+    $fh->print($self->{' DAT'});
+    $self->{' OUTLEN'} = length($self->{' DAT'});
     $self;
 }
 
 
+=head2 $g->out_xml($context, $depth)
+
+Outputs an XML description of the glyph
+
+=cut
+
+sub out_xml
+{
+    my ($self, $context, $depth) = @_;
+    my ($addr) = ($self =~ m/\((.+)\)$/o);
+    my ($k, $ndepth);
+
+    if ($context->{'addresses'}{$addr})
+    {
+        $context->{'fh'}->printf("%s<glyph gid='%s' id_ref='%s'/>\n", $depth, $context->{'gid'}, $addr);
+        return $self;
+    }
+    else
+    {
+        $context->{'fh'}->printf("%s<glyph gid='%s' id='%s'>\n", $depth, $context->{'gid'}, $addr);
+    }
+    
+    $ndepth = $depth . $context->{'indent'};
+    $self->read_dat;
+    foreach $k (sort grep {$_ !~ m/^\s/o} keys %{$self})
+    {
+        $self->XML_element($context, $ndepth, $k, $self->{$k});
+    }
+    $context->{'fh'}->print("$depth</glyph>\n");
+    delete $context->{'done_points'};
+    $self;
+}
+    
+
+sub XML_element
+{
+    my ($self, $context, $depth, $key, $val) = @_;
+    my ($fh) = $context->{'fh'};
+    my ($dind) = $depth . $context->{'indent'};
+    my ($i);
+    
+    if ($self->{'numberOfContours'} >= 0 && ($key eq 'x' || $key eq 'y' || $key eq 'flags'))
+    {
+        return $self if ($context->{'done_points'});
+        $context->{'done_points'} = 1;
+
+        $fh->print("$depth<points>\n");
+        for ($i = 0; $i <= $#{$self->{'flags'}}; $i++)
+        { $fh->printf("%s<point x='%s' y='%s' flags='0x%02X'/>\n", $dind,
+                $self->{'x'}[$i], $self->{'y'}[$i], $self->{'flags'}[$i]); }
+        $fh->print("$depth</points>\n");
+    }
+    elsif ($key eq 'hints')
+    {
+        my ($dat);
+        $fh->print("$depth<hints>\n");
+#        Font::TTF::Utils::XML_hexdump($context, $depth . $context->{'indent'}, $self->{'hints'});
+        $dat = Font::TTF::Utils::XML_binhint($self->{'hints'});
+        $dat =~ s/\n(?!$)/\n$depth$context->{'indent'}/mg;
+        $fh->print("$depth$context->{'indent'}$dat");
+        $fh->print("$depth</hints>\n");
+    }
+    else
+    { return Font::TTF::Table::XML_element(@_); }
+
+    $self;    
+}
+
+
 =head2 $g->update
 
 Generates a C<$self->{'DAT'}> from the internal structures, if the data has
@@ -413,14 +483,14 @@ sub update
 
     return $self unless (defined $self->{' read'} && $self->{' read'} > 1);
     $self->update_bbox;
-    $self->{'DAT'} = TTF_Out_Fields($self, \%fields, 10);
+    $self->{' DAT'} = TTF_Out_Fields($self, \%fields, 10);
     $num = $self->{'numberOfContours'};
     if ($num > 0)
     {
-        $self->{'DAT'} .= pack("n*", @{$self->{'endPoints'}});
+        $self->{' DAT'} .= pack("n*", @{$self->{'endPoints'}});
         $len = $self->{'instLen'};
-        $self->{'DAT'} .= pack("n", $len);
-        $self->{'DAT'} .= pack("a" . $len, substr($self->{'hints'}, 0, $len));
+        $self->{' DAT'} .= pack("n", $len);
+        $self->{' DAT'} .= pack("a" . $len, substr($self->{'hints'}, 0, $len)) if ($len > 0);
         for ($i = 0; $i < $self->{'numPoints'}; $i++)
         {
             $flag = $self->{'flags'}[$i] & 1;
@@ -445,7 +515,7 @@ sub update
                 $flag |= 4;
                 $flag |= 32 if ($y >= 0);
             }
-            $self->{'DAT'} .= pack("C", $flag);                    # sorry no repeats
+            $self->{' DAT'} .= pack("C", $flag);                    # sorry no repeats
             $self->{'flags'}[$i] = $flag;
         }
         for ($i = 0; $i < $self->{'numPoints'}; $i++)
@@ -453,22 +523,22 @@ sub update
             $flag = $self->{'flags'}[$i];
             $x = $self->{'x'}[$i] - (($i == 0) ? 0 : $self->{'x'}[$i - 1]);
             if (($flag & 18) == 0)
-            { $self->{'DAT'} .= TTF_Pack("s", $x); }
+            { $self->{' DAT'} .= TTF_Pack("s", $x); }
             elsif (($flag & 18) == 18)
-            { $self->{'DAT'} .= pack("C", $x); }
+            { $self->{' DAT'} .= pack("C", $x); }
             elsif (($flag & 18) == 2)
-            { $self->{'DAT'} .= pack("C", -$x); }
+            { $self->{' DAT'} .= pack("C", -$x); }
         }
         for ($i = 0; $i < $self->{'numPoints'}; $i++)
         {
             $flag = $self->{'flags'}[$i];
             $y = $self->{'y'}[$i] - (($i == 0) ? 0 : $self->{'y'}[$i - 1]);
             if (($flag & 36) == 0)
-            { $self->{'DAT'} .= TTF_Pack("s", $y); }
+            { $self->{' DAT'} .= TTF_Pack("s", $y); }
             elsif (($flag & 36) == 36)
-            { $self->{'DAT'} .= pack("C", $y); }
+            { $self->{' DAT'} .= pack("C", $y); }
             elsif (($flag & 36) == 4)
-            { $self->{'DAT'} .= pack("C", -$y); }
+            { $self->{' DAT'} .= pack("C", -$y); }
         }
     }
 
@@ -498,31 +568,32 @@ sub update
             else
             { $flag |= 32; }
             
-            $self->{'DAT'} .= pack("n", $flag);
-            $self->{'DAT'} .= pack("n", $comp->{'glyph'});
+            $self->{' DAT'} .= pack("n", $flag);
+            $self->{' DAT'} .= pack("n", $comp->{'glyph'});
             $comp->{'flag'} = $flag;
 
             if ($flag & 1)
-            { $self->{'DAT'} .= TTF_Pack("s2", @{$comp->{'args'}}); }
+            { $self->{' DAT'} .= TTF_Pack("s2", @{$comp->{'args'}}); }
             else
-            { $self->{'DAT'} .= pack("CC", @{$comp->{'args'}}); }
+            { $self->{' DAT'} .= pack("CC", @{$comp->{'args'}}); }
 
             if ($flag & 8)
-            { $self->{'DAT'} .= TTF_Pack("F", $comp->{'scale'}[0]); }
+            { $self->{' DAT'} .= TTF_Pack("F", $comp->{'scale'}[0]); }
             elsif ($flag & 64)
-            { $self->{'DAT'} .= TTF_Pack("F2", $comp->{'scale'}[0], $comp->{'scale'}[3]); }
+            { $self->{' DAT'} .= TTF_Pack("F2", $comp->{'scale'}[0], $comp->{'scale'}[3]); }
             elsif ($flag & 128)
-            { $self->{'DAT'} .= TTF_Pack("F4", @{$comp->{'scale'}}); }
+            { $self->{' DAT'} .= TTF_Pack("F4", @{$comp->{'scale'}}); }
         }
         if (defined $self->{'instLen'} && $self->{'instLen'} > 0)
         {
             $len = $self->{'instLen'};
-            $self->{'DAT'} .= pack("n", $len);
-            $self->{'DAT'} .= pack("a" . $len, substr($self->{'hints'}, 0, $len));
+            $self->{' DAT'} .= pack("n", $len);
+            $self->{' DAT'} .= pack("a" . $len, substr($self->{'hints'}, 0, $len));
         }
     }
-    $self->{'DAT'} .= "\000" if (length($self->{'DAT'}) & 1);
-    $self->{'OUTLEN'} = length($self->{'DAT'});
+    $self->{' DAT'} .= "\000" if (length($self->{' DAT'}) & 1);
+    $self->{' OUTLEN'} = length($self->{' DAT'});
+    $self->{' read'} = 1;
 # we leave numPoints and instLen since maxp stats use this
     $self;
 }
@@ -540,7 +611,7 @@ sub update_bbox
     my ($num, $maxx, $minx, $maxy, $miny, $i, $comp, $x, $y, $compg);
 
     return $self unless $self->{' read'} > 1;       # only if read_dat done
-    $miny = $minx = 65537; $maxx = $maxy = 0;
+    $miny = $minx = 65537; $maxx = $maxy = -65537;
     $num = $self->{'numberOfContours'};
     if ($num > 0)
     {
@@ -590,8 +661,6 @@ sub update_bbox
     $self->{'yMin'} = $miny;
     $self;
 }
-            
-
 
             
 =head2 $g->maxInfo
@@ -610,8 +679,8 @@ sub maxInfo
     $res[4] = length($self->{'hints'}) if defined $self->{'hints'};
     if ($self->{'numberOfContours'} > 0)
     {
-        $res[0] = $self->{'numPoints'};
-        $res[1] = $self->{'numberOfContours'};
+        $res[2] = $res[0] = $self->{'numPoints'};
+        $res[3] = $res[1] = $self->{'numberOfContours'};
         $res[6] = 1;
     } elsif ($self->{'numberOfContours'} < 0)
     {
@@ -638,13 +707,80 @@ Useful for saving memory in apps with many glyphs being read
 sub empty
 {
     my ($self) = @_;
-    my (%keep) = map {$_ => 1} ('LOC', 'OUTLOC', ' PARENT', 'INFILE', 'BASE',
+    my (%keep) = map {(" $_" => 1)} ('LOC', 'OUTLOC', 'PARENT', 'INFILE', 'BASE',
                                 'OUTLEN', 'LEN');
     map {delete $self->{$_} unless $keep{$_}} keys %$self;
     
     $self;
 }
-        
+
+
+=head2 $g->get_points
+
+This method creates point information for a compound glyph. The information is
+stored in the same place as if the glyph was not a compound, but since
+numberOfContours is negative, the glyph is still marked as being a compound
+
+=cut
+
+sub get_points
+{
+    my ($self) = @_;
+    my ($comp, $compg, $nump, $e, $i);
+
+    $self->read_dat;
+    return undef unless ($self->{'numberOfContours'} < 0);
+
+    foreach $comp (@{$self->{'comps'}})
+    {
+        $compg = $self->{' PARENT'}{'loca'}{'glyphs'}[$comp->{'glyph'}]->read;
+        $compg->get_points;
+
+        for ($i = 0; $i < $compg->{'numPoints'}; $i++)
+        {
+            my ($x, $y) = ($compg->{'x'}[$i], $compg->{'y'}[$i]);
+            if (defined $comp->{'scale'})
+            {
+                ($x, $y) = ($x * $comp->{'scale'}[0] + $y * $comp->{'scale'}[2],
+                            $x * $comp->{'scale'}[1] + $y * $comp->{'scale'}[3]);
+            }
+            if (defined $comp->{'args'})
+            { ($x, $y) = ($x + $comp->{'args'}[0], $y + $comp->{'args'}[1]); }
+            push (@{$self->{'x'}}, $x);
+            push (@{$self->{'y'}}, $y);
+        }
+        foreach $e (@{$compg->{'endPoints'}})
+        { push (@{$self->{'endPoints'}}, $e + $nump); }
+        $nump += $compg->{'numPoints'};
+    }
+    $self->{'numPoints'} = $nump;
+    $self;
+}
+
+
+=head2 $g->get_refs
+
+Returns an array of all the glyph ids that are used to make up this glyph. That
+is all the compounds and their references and so on. If this glyph is not a
+compound, then returns an empty array
+
+=cut
+
+sub get_refs
+{
+    my ($self) = @_;
+    my (@res, $g);
+
+    $self->read_dat;
+    return unless ($self->{'numberOfContours'} < 0);
+    foreach $g (@{$self->{'comps'}})
+    {
+        my (@list) = $self->{' PARENT'}{'loca'}{'glyphs'}[$g->{'glyph'}]->get_points;
+        push (@res, $g->{'glyph'});
+        push (@res, @list) if ($list[0]);
+    }
+    return @res;
+}
 
 1;
 
